<?php namespace EscapeWork\Manager\Banners\Models;

use EscapeWork\Manager\Banners\Repositories\BannerRepository;
use EscapeWork\Manager\Banners\Extensions\BannerCollection;
use EscapeWork\LaravelHelpers\Eloquent;
use EscapeWork\Resize\Resize;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Config;
use Str;

class Banner extends Eloquent implements BannerRepository
{

    /**
     * Tabela
     */
    protected $table = 'banners';

    /**
     * Fillable fields
     */
    public $fillable = array(
        'id',
        'banner_group_id',
        'title',
        'description',
        'link',
        'target',
        'order',
        'active',
        'expire_at',
        'created_at',
        'updated_at',
    );

    /**
     * Tipos de imagens aceitas
     */
    public $acceptedImages = array(
        'image/jpg', 'image/jpeg', 'image/pjpg', 'image/pjpeg',
        'image/png'
    );

    /**
     * Diretórios
     */
    public
        $dir = 'files/banners',
        $url = 'files/banners';

    /**
     * Setando o link
     */
    public function setLinkAttribute($value)
    {
        if (filter_var($value, FILTER_VALIDATE_URL)) {
            $this->attributes['link'] = $value;
        } else {
            $value = 'http://' .$value;

            if (filter_var($value, FILTER_VALIDATE_URL)) {
                $this->attributes['link'] = $value;
            }
        }
    }

    /**
     * Setting the date attribute
     */
    public function setExpireAtAttribute($value)
    {
        $this->attributes['expire_at'] = to_timestamp($value, 'd/m/Y', 'Y-m-d');
    }

    /**
     * Setando o attributo de ativo
     */
    public function setActiveAttribute($value)
    {
        $this->attributes['active'] = $value == 1 ? 1 : 0;
    }

    /**
     * Setando a descrição
     */
    public function setDescriptionAttribute($value)
    {
        if ($value != '') {
            $this->attributes['description'] = $value;
        }
    }

    /**
     * Setando o target
     */
    public function setTargetAttribute($value)
    {
        $this->attributes['target'] = $value == '_blank' ? '_blank' : '_self';
    }

    /**
     * Active Scope
     */
    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }

    /**
     * Not Expired scope
     */
    public function scopeNotExpired($query)
    {
        return $query->where(function($query)
        {
            $query->whereNull('expire_at')
                  ->orWhere('expire_at', '>=', date('Y-m-d'));
        });
    }

    /**
     * Group relashionship
     */
    public function group()
    {
        return $this->belongsTo('EscapeWork\Manager\Banners\Models\BannerGroup', 'banner_group_id');
    }

    /**
     * Returning the active banners
     */
    public function getActives()
    {
        return $this->active()
                    ->notExpired()
                    ->orderBy('order', 'asc')
                    ->get();
    }

    /**
     * Retornando a URL da imagem
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * Retornando a URL da imagem
     */
    public function getDir()
    {
        return public_path() . '/' . $this->dir;
    }

    /**
     * Fazendo o upload do banner
     */
    public function uploadBanner($banner, $update = false)
    {
        if (! $banner instanceof UploadedFile) {
            static::$messages[] = 'Selecione um arquivo';
            return false;
        }

        # validando o diretório
        if (! $this->_makeDirectory()) {
            static::$messages[] = 'O diretório <strong>'.$this->getDir.'</strong> precisa de permissão de escrita para salvar os banners';

            return false;
        }

        # validando o tipo
        if (! $this->_validateBannerType($banner->getMimeType())) {
            static::$messages[] = 'Apenas imagens <strong>jpg e png</strong> são aceitas';

            return false;
        }

        # formatando o nome
        $dir      = $this->getDir();
        $filename = $this->_formatName($banner->getClientOriginalName(), $dir, $banner->guessExtension());

        # copiando a imagem
        try {
            $banner = $banner->move($dir . '/', $filename);
            $this->_createSizes($dir, $filename);

            # excluindo o arquivo atual
            if ($update) {
                $this->_deleteBanner();
            }

            $this->banner = $filename;

            return true;
        } catch(Exception $e) {
            return false;
        }
    }

    /**
     * Validando o tipo da imagem
     */
    public function _validateBannerType($type)
    {
        return in_array($type, $this->acceptedImages);
    }

    /**
     * Formatando o nome
     */
    protected function _formatName($name, $dir, $extension)
    {
        $name    = explode('.', $name);
        $total   = count($name);

        if ($total > 1) {
            unset($name[$total - 1]);
        }

        $name     = implode('', $name);
        $filename = Str::slug($name) . '.' . $extension;

        while (is_file($dir . '/' . $filename)) {
            $filename = '_' . $filename;
        }

        return $filename;
    }

    /**
     * Fazendo o resize da imagem
     *
     * @throws  EscapeWork\Resize\ResizeException
     */
    public function _createSizes($dir, $filename)
    {
        $sizes = Config::get('manager-banners::sizes');
        $sizes = isset($sizes[$this->banner_group_id]) ? $sizes[$this->banner_group_id] : $sizes['default'];

        foreach ($sizes as $prefix => $size) {
            $prefix = $prefix == 'default' ? '' : $prefix . '-';
            $banner = $dir . '/' . $prefix . $filename;

            # copiando o arquivo
            if (! is_file($banner)) {
                copy($dir . '/' . $filename, $banner);
            }

            # resize
            $resize = new Resize($banner);
            $resize->setWidth($size['width'])->setHeight($size['height'])->crop();
        }
    }

    /**
     * Excluindo o arquivo antigo
     */
    public function _deleteBanner()
    {
        $dir  = $this->getDir();
        $file = $dir . '/' . $this->banner;

        if (is_file($file)) {
            unlink($file);
        }
    }

    /**
     * Criando o diretório caso não existe
     */
    public function _makeDirectory()
    {
        $dir = $this->getDir();

        if (! is_dir($dir)) {
            if (mkdir($dir, 0773)) {
                chmod($dir, 0773);
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Excluindo o banner
     */
    public function delete()
    {
        $this->_deleteBanner();

        return parent::delete();
    }

    /**
     * Nova coleção
     */
    public function newCollection(array $models = array())
    {
        return new BannerCollection($models);
    }
}
